<?php
/**
 * Copyright Â© 2015 FiloBlu . All rights reserved.
 */

namespace FiloBlu\Flow\Helper;

use Exception;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\ProductRepository;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Module\Manager;
use Magento\Store\Model\ScopeInterface;
use Monolog\Logger;

/**
 * Class Product
 * @package FiloBlu\Flow\Helper
 */
class Product extends AbstractHelper
{
    const CONFIG_FLOW_ATTRIBUTE_PATH = 'filoblu_flow/flow_process/flow_attribute_select';
    const CONFIG_FLOW_ONLYCONFIGURABLE_PATH = 'filoblu_flow/flow_process/flow_only_configurable_category';
    const CONFIG_FLOW_UPDATECATEGORY_PATH = 'filoblu_flow/flow_process/flow_update_category';
    const CONFIG_FLOW_MERGE_PATH = 'filoblu_flow/flow_process/flow_merge_category';
    const CONFIG_FLOW_ASSIGN_ROOT_CAT = 'filoblu_flow/flow_process/flow_assign_root_categories';
    const CONFIG_FLOW_ASSIGN_FULL_TREE_PATH = 'filoblu_flow/flow_process/flow_assign_full_tree_category';
    const CONFIG_FLOW_UPDATE_CONFIGURABLE_URL = 'filoblu_flow/flow_process/flow_configurable_url';
    const CONFIG_FLOW_ADD_TO_ALL_WEBSITES = 'filoblu_flow/flow_ite/add_to_websites';
    const CONFIG_FLOW_ITE_COPY_NAME_TO_DESCRIPTION = 'filoblu_flow/flow_ite/copy_name_in_description';
    const CONFIG_FLOW_ITE_COPY_NAME_TO_SHORT_DESCRIPTION = 'filoblu_flow/flow_ite/copy_name_in_short_description';
    const CONFIG_FLOW_ITE_DISABLE_SHOP_STATUS = 'filoblu_flow/flow_ite/disable_shop_status';
    const CONFIG_FLOW_ITE_HIDE_PRICE_STATUS = 'filoblu_flow/flow_ite/hide_price_status';
    const CONFIG_FLOW_INHERITCATOFSIMPLE_PATH = 'filoblu_flow/flow_process/flow_inherit_categories_of_simple';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var ProductMetadataInterface
     */
    protected $_productMetadata;

    /**
     * @var ResourceConnection
     */
    protected $_resource;

    /**
     * @var
     */
    protected $connection;

    /**
     * @var Logger|Logger
     */
    protected $_logger;

    /**
     * @var ProductRepository
     */
    protected $_productRepository;

    /**
     * @var Manager
     */
    protected $_moduleManager;

    /**
     * @var CollectionFactory
     */
    protected $productCollectionFactory;

    /**
     * Product constructor.
     * @param Context $context
     * @param ProductMetadataInterface $_productMetadata
     * @param ResourceConnection $resource
     * @param LoggerProvider $loggerProvider
     * @param CollectionFactory $productCollectionFactory
     * @param ProductRepository $productRepository
     * @param Manager $moduleManager
     */
    public function __construct(
        Context $context,
        ProductMetadataInterface $_productMetadata,
        ResourceConnection $resource,
        LoggerProvider $loggerProvider,
        CollectionFactory $productCollectionFactory,
        ProductRepository $productRepository,
        Manager $moduleManager
    )
    {
        $this->scopeConfig = $context->getScopeConfig();
        $this->_resource = $resource;
        $this->_productMetadata = $_productMetadata;
        $this->_logger = $loggerProvider->getLogger();
        $this->_productRepository = $productRepository;
        $this->_moduleManager = $moduleManager;
        $this->productCollectionFactory = $productCollectionFactory;
        parent::__construct($context);
    }

    /**
     * Import categories only on configurables?
     * @return mixed
     */
    public function getOnlyConfigurableCategoryFlag()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ONLYCONFIGURABLE_PATH, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * If a simple is passed do i need to set the same categories on parents?
     * @return mixed
     */
    public function getParentInheritCategoriesOfSimple()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_INHERITCATOFSIMPLE_PATH, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * Is process categories enabled?
     * @return mixed
     */
    public function processCategoriesEnabledFlag()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_UPDATECATEGORY_PATH, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * Categories need to be merged with the old ones?
     * @return mixed
     */
    public function mergeCategoriesFlag()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_MERGE_PATH, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * Assign product also on root categories? (level 1 categories on database)
     * @return mixed
     */
    public function assignRootCategories()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ASSIGN_ROOT_CAT, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * Do I need to assign the product to the full category tree?
     * @return mixed
     */
    public function assignFullTree()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ASSIGN_FULL_TREE_PATH, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getUpdateConfigurableUrl()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_UPDATE_CONFIGURABLE_URL, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getCopyNameInDescription()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_COPY_NAME_TO_DESCRIPTION, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getCopyNameInShortDescription()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_COPY_NAME_TO_SHORT_DESCRIPTION, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function isDisableShopModuleEnabled()
    {
        return $this->_moduleManager->isEnabled('FiloBlu_DisableShop');
    }

    /**
     * @return mixed
     */
    public function getDisableShopStatus()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_DISABLE_SHOP_STATUS, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getHidePriceStatus()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_HIDE_PRICE_STATUS, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return int
     * @throws Exception
     * @deprecated
     */
    public function getProductAttributeSet()
    {
        throw new Exception(__CLASS__ . '::' . __METHOD__ . ' Avoid using this deprecated method');
        return 4;
    }

    /**
     * @return bool
     */
    public function createNewProducts()
    {
        return true;
    }

    /**
     * @return int
     */
    public function getInitialProductStatus()
    {
        return Status::STATUS_DISABLED;
    }

    /**
     * @return mixed
     */
    public function getAddToWebsites()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ADD_TO_ALL_WEBSITES);
    }

    /**
     * @param $TaxClassCode
     *
     * @return int
     * @throws Exception
     * @deprecated
     */
    public function getMappedTaxClassId($TaxClassCode)
    {
        throw new Exception(__CLASS__ . '::' . __METHOD__ . ' Avoid using this deprecated method');
        return 2;
    }

    /**
     * @param $childProducts
     * @return mixed
     * @throws Exception
     */
    public function getParentProductsIds($childProducts)
    {
        $relationTableName = $this->getConnection()->getTableName('catalog_product_super_link');

        $products = $this->productCollectionFactory->create();

        $products->getSelect()->join(['super_link' => $relationTableName], 'e.entity_id = super_link.parent_id');
        $products->getSelect()->where('super_link.product_id IN (?)', $childProducts);
        //$products->addFieldToFilter('type_id', $productType);
        $products->getSelect()->group('e.entity_id');

        $result = $products->getAllIds();

        // Need to convert row_id into entity_id
        if ($this->isEnterpriseEdition() && count($result)) {
            foreach ($result as $key => $value) {

                if (!$value) {
                    continue;
                }

                try {
                    $entity_id = $this->getConnection()->fetchRow('SELECT entity_id FROM catalog_product_entity WHERE row_id = ?', [$value]);
                } catch (Exception $e) {
                    $message = "Can't find entity_id from row_id '{$value}')";
                    $this->_logger->info($message);
                    throw new Exception($e->getMessage());
                }

                if (isset($entity_id['entity_id']) && $entity_id['entity_id'] > 0) {
                    $result[$key] = $entity_id['entity_id'];
                }

            }
        }

        return $result;
    }

    /**
     * @return AdapterInterface
     */
    public function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->_resource->getConnection();
        }
        return $this->connection;
    }

    /**
     * @return bool
     */
    protected function isEnterpriseEdition()
    {
        return ($this->_productMetadata->getEdition() === 'Enterprise') ? true : false;
    }

    /**
     * @return array
     */
    public function getSimpleToConfigurableProductAttributesToExclude()
    {
        $saved = [];

        if (is_string($saved)) {
            return explode(',', $saved);
        }

        return $saved;
    }

    /**
     * @return array
     */
    public function getSimpleToConfigurableSyncAttributes()
    {
        $attributesToCopy = $this->scopeConfig->getValue(self::CONFIG_FLOW_ATTRIBUTE_PATH, ScopeInterface::SCOPE_WEBSITE);

        if (!empty($attributesToCopy)) {
            return explode(',', $attributesToCopy);
        }

        return [];
    }

    /**
     * @return bool
     */
    public function getUpdateConfigurablePrice()
    {
        return false;
    }

    /**
     * Get Configurable Childs by Sku
     *
     * @param $sku
     *
     * @return array
     */
    public function getConfigurableChildsBySku($sku)
    {
        $simples = [];

        try {
            // Get Product
            $product = $this->_productRepository->get($sku);

            //If product is configurable
            if ($product->getTypeId() === 'configurable') {
                $attributes = $product->getTypeInstance()->getConfigurableOptions($product);
                //Get childs for super attribute
                foreach ($attributes as $attribute) {
                    foreach ($attribute as $child) {
                        $simples[] = $child['sku'];
                    }
                }
            } //if is simple
            else {
                $simples[] = $sku;
            }
        } catch (Exception $e) {
            $simples = [];
            $this->_logger->info($e->getMessage() . ' for sku ' . $sku);
        }

        return $simples;
    }
}