<?php
/**
 * File:   Addtodefaultcategory.php
 * Author: Ernesto Minicone (napoli1890)
 * Date:   15/06/17
 * Time:   16.27
 */

namespace FiloBlu\Flow\Model\Actions\From;

use Exception;
use FiloBlu\Flow\Helper\LoggerProvider;
use FiloBlu\Flow\Helper\Product;
use Magento\Catalog\Model\CategoryFactory;
use Magento\Catalog\Model\ProductFactory;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Addtodefaultcategory
 * @package FiloBlu\Flow\Model\Actions\From
 */
class Addtodefaultcategory extends Addtocategory
{
    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * Addtodefaultcategory constructor.
     * @param Product $helper_product
     * @param LoggerProvider $loggerProvider
     * @param CategoryFactory $categoryFactory
     * @param ProductFactory $productFactory
     * @param Configurable $configurableProduct
     * @param StoreManagerInterface $_storeManager
     * @param CollectionFactory $categoryCollectionFactory
     */
    public function __construct(
        Product $helper_product,
        LoggerProvider $loggerProvider,
        CategoryFactory $categoryFactory,
        ProductFactory $productFactory,
        Configurable $configurableProduct,
        StoreManagerInterface $_storeManager,
        CollectionFactory $categoryCollectionFactory
    ) {
        $this->storeManager = $_storeManager;
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($helper_product, $loggerProvider, $categoryFactory, $productFactory, $configurableProduct, $categoryCollectionFactory);
    }

    /**
     * @param $product
     * @param $value
     * @param $storeId
     * @return bool|void
     * @throws Exception
     */
    public function process($product, $value, $storeId)
    {
        $this->_logger->addInfo('Processing Action addtodefaultcategory for product: ' . $product->getId());

        $storeManager = $this->storeManager;
        $stores = $storeManager->getStores();

        // For each store, get default category
        $default_categories = [];
        /** @var $store Store */
        foreach ($stores as $store) {
            $default_categories[] = $store->getRootCategoryId();
        }

        // Filter categories
        $default_categories = array_unique(array_filter($default_categories));

        // Workaround on Addtocategory
        if (count($default_categories) === 1) {
            $default_categories[] = $default_categories[0];
        }

        // Create list of category ids
        if ($default_categories) {
            $value = implode('|', $default_categories);

            // Set value as $rootCatId | $rootCatId as workaround on Addtocategory
            parent::process($product, $value, $storeId);
            return;
        } else {
            throw new Exception('No root categories configured on stores');
        }
    }
}
