<?php

namespace FiloBlu\Flow\Model\System;

use Magento\Catalog\Model\Product;
use Magento\Eav\Model\Entity\Type;
use Magento\Eav\Model\ResourceModel\Entity\Attribute\Collection;
use Magento\Framework\App\ObjectManager;

/**
 * Class ProductAttributes
 * @package FiloBlu\Flow\Model\System
 */
class ProductAttributes
{

    /**
     * @return array
     */
    public function toOptionArray()
    {
        $options = [];
        $attributes = $this->getAttributesToCopy();
        $attributeGroups = array_unique($attributes->getColumnValues('attribute_group_name'));
        foreach ($attributeGroups as $attributeGroupName) {
            $groupedOptions = [];
            $groupAttributes = $attributes->getItemsByColumnValue('attribute_group_name', $attributeGroupName);
            foreach ($groupAttributes as $attribute) {
                $attributeLabel = $attribute->getFrontendLabel() ?: $attribute->getAttributeCode();
                $groupedOptions[] = [
                    'label' => $attributeLabel,
                    'value' => $attribute->getAttributeCode()
                ];
            }

            $options[] = [
                'label' => $attributeGroupName,
                'value' => $groupedOptions
            ];
        }

        return $options;
    }

    /**
     * @return mixed
     */
    public function getAttributesToCopy()
    {
        $excludedGroups = [
            'Prices',
            'Recurring Profile'
        ];

        $excludedAttributes = [
            'sku',
            'status',
            'visibility',
            'old_id',
// 			'url_key',
// 			'url_path',
            'has_options',
            'required_options',
            'options_container',
            'created_at',
            'updated_at',
            'category_ids'
        ];

        $excludedFrontendInputs = [
            'price',
            'image',
            'media_image',
            'gallery',
            'filter_price_range',
            'price',
            'special_price',
            'tier_price',
            'minimal_price',
            'price_type',
            'price_view',
            'msrp_display_actual_price_type',
            'gift_wrapping_price',
            'hide_price'
        ];

        $productEntityType = ObjectManager::getInstance()->create(Type::class)->loadByCode(Product::ENTITY);
        $attributes = ObjectManager::getInstance()->create(Collection::class);
        $attributes->setEntityTypeFilter($productEntityType);
//        $attributes = Mage::getResourceModel('eav/entity_attribute_collection')->setEntityTypeFilter($productEntityType);
        $attributes->join(['eea' => 'eav_entity_attribute'], 'main_table.attribute_id = eea.attribute_id', ['group_id' => 'eea.attribute_group_id']);
        $attributes->join(['eeg' => 'eav_attribute_group'], 'eea.attribute_group_id = eeg.attribute_group_id', ['attribute_group_name' => 'eeg.attribute_group_name']);
//        $attributes->addFieldToFilter(
//            array('apply_to','apply_to','apply_to',),
//            array(
//                array("eq" => 'NULL'),
//                array("eq" => ''),
//                array("like" => '%'.\Magento\Catalog\Model\Product\Type::TYPE_SIMPLE.'%')
//            )
//        );

//        $attributes->where(sprintf("(apply_to IS NULL OR apply_to = '' OR apply_to LIKE '%s')", '%'.\Magento\Catalog\Model\Product\Type::TYPE_SIMPLE.'%'));
        $attributes->addFieldToFilter('attribute_group_name', ['nin' => $excludedGroups]);
        $attributes->addFieldToFilter('frontend_input', ['nin' => $excludedFrontendInputs]);
        $attributes->addFieldToFilter('attribute_code', ['nin' => $excludedAttributes]);
        $attributes->getSelect()->group('main_table.attribute_id');
        $attributes->getSelect()->order('attribute_group_name ASC');
        return $attributes;
    }
}
